@@ -192,9 +192,7 @@ impl AteccDevice {
 
             start_pos += shift;
             let remaining_bytes = data.len() - start_pos;
-            if 0 == remaining_bytes {
-                shift = 0
-            } else if remaining_bytes < ATCA_AES_DATA_SIZE {
+            if remaining_bytes < ATCA_AES_DATA_SIZE {
                 shift = remaining_bytes
             }
         }
@@ -294,9 +292,7 @@ impl AteccDevice {
 
             start_pos += shift;
             let remaining_bytes = data.len() - start_pos;
-            if 0 == remaining_bytes {
-                shift = 0
-            } else if remaining_bytes < ATCA_AES_DATA_SIZE {
+            if remaining_bytes < ATCA_AES_DATA_SIZE {
                 shift = remaining_bytes
             }
         }
